import React from 'react';
import PropTypes from 'prop-types';
import { TokenStream } from 'prismjs';

interface IToken {
    types: string[];
    content: string;
    empty?: boolean;
}
interface ILineInputProps {
    key?: React.Key;
    style?: React.CSSProperties;
    className?: string;
    line: IToken[];
    [otherProp: string]: any;
}
interface ILineOutputProps {
    key?: React.Key;
    style?: React.CSSProperties;
    className: string;
    [otherProps: string]: any;
}
interface ITokenInputProps {
    key?: React.Key;
    style?: React.CSSProperties;
    className?: string;
    token: IToken;
    [otherProp: string]: any;
}
interface ITokenOutputProps {
    key?: React.Key;
    style?: React.CSSProperties;
    className: string;
    children: string;
    [otherProp: string]: any;
}
interface IHighlightTokens {
    tokens: IToken[][];
    className?: string;
    style: React.CSSProperties;
    getLineProps(input: ILineInputProps): ILineOutputProps;
    getTokenProps(input: ITokenInputProps): ITokenOutputProps;
}
interface PrismThemeEntry {
    color?: string;
    backgroundColor?: string;
    fontStyle?: 'normal' | 'italic';
    fontWeight?: 'normal' | 'bold' | '100' | '200' | '300' | '400' | '500' | '600' | '700' | '800' | '900';
    textDecorationLine?: 'none' | 'underline' | 'line-through' | 'underline line-through';
    opacity?: number;
    [styleKey: string]: string | number | void;
}
interface IPrismTheme {
    plain: PrismThemeEntry;
    styles: Array<{
        types: string[];
        style: PrismThemeEntry;
        languages?: string[];
    }>;
}
interface IThemeDict {
    root: React.CSSProperties;
    plain: React.CSSProperties;
    [type: string]: React.CSSProperties;
}

interface ICodeHighlighterProps {
    /**
     * Source code contents
     */
    value: string;
    /**
     * Code language
     */
    lang?: string | null;
    /**
     * Line number of Lines that should be highlighted.
     */
    highlightLinenos?: number[];
    /**
     * Whether the code block is in a collapsed state.
     * @default false
     */
    collapsed?: boolean;
    /**
     * Maximum number of rows displayed
     * @default -1
     */
    maxLines?: number;
    /**
     * Line height.
     * @default '1.6rem'
     */
    lineHeight?: React.CSSProperties['lineHeight'];
    /**
     * Whether should display line numbers.
     * @default true
     */
    showLineNo?: boolean;
    /**
     * If true, use vscDarkTheme as default theme,
     * otherwise use vscLightTheme as default theme.
     * @default true
     */
    darken?: boolean;
    /**
     * Highlight prism theme.
     */
    theme?: IPrismTheme;
    /**
     * Ref of the codes area.
     */
    codesRef?: React.RefCallback<HTMLDivElement> | React.RefObject<HTMLDivElement>;
    /**
     * Callback when linenoWidth changed.
     */
    onLinenoWidthChange?(linenoWidth: React.CSSProperties['width']): void;
}
declare const CodeHighlighter: React.FC<ICodeHighlighterProps>;

interface IHighlightContentProps {
    /**
     * Source literal code.
     */
    code: string;
    /**
     * Ref of the codes area.
     */
    codesRef: React.RefCallback<HTMLDivElement> | React.RefObject<HTMLDivElement> | undefined;
    /**
     * Whether the code block is in a collapsed state.
     */
    collapsed: boolean;
    /**
     * Line number of Lines that should be highlighted.
     */
    highlightLinenos: number[] | undefined;
    /**
     * Code language.
     */
    lang: string | undefined;
    /**
     * Line height.
     */
    lineHeight: React.CSSProperties['lineHeight'];
    /**
     * Maximum number of rows displayed
     */
    maxLines: number;
    /**
     * Whether should display line numbers.
     */
    showLineNo: boolean;
    /**
     * Highlight prism theme.
     */
    theme: IPrismTheme;
    /**
     * Callback when linenoWidth changed.
     */
    onLinenoWidthChange?(linenoWidth: React.CSSProperties['width']): void;
}
declare const HighlightContent: React.FC<IHighlightContentProps>;

interface IHighlightLinenosProps {
    countOfLines: number;
    highlightLinenos: number[] | undefined;
}
declare class HighlightLinenos extends React.Component<IHighlightLinenosProps> {
    static propTypes: {
        countOfLines: PropTypes.Validator<number>;
        highlightLinenos: PropTypes.Requireable<any[]>;
    };
    render(): React.ReactElement;
    shouldComponentUpdate(nextProps: Readonly<IHighlightLinenosProps>): boolean;
}

interface IHighlightTokensProps {
    tokens: IToken[][];
    highlightLinenos: number[] | undefined;
    getLineProps(input: ILineInputProps): ILineOutputProps;
    getTokenProps(input: ITokenInputProps): ITokenOutputProps;
}
declare class HighlightTokens extends React.Component<IHighlightTokensProps> {
    static propTypes: {
        tokens: PropTypes.Validator<any[]>;
        highlightLinenos: PropTypes.Requireable<any[]>;
        getLineProps: PropTypes.Validator<(...args: any[]) => any>;
        getTokenProps: PropTypes.Validator<(...args: any[]) => any>;
    };
    render(): React.ReactElement;
    shouldComponentUpdate(nextProps: Readonly<IHighlightTokensProps>): boolean;
}

declare const useHighlightTokens: (code: string, language: string, theme: IPrismTheme) => IHighlightTokens;

/**
 * Generated Prism Theme from VSCode .json Themes
 *
 * @see https://github.com/FormidableLabs/prism-react-renderer/blob/95025358684be04865669bb912e8cf6203c1a391/tools/themeFromVsCode/README.md
 */
declare const vscDarkTheme: IPrismTheme;

/**
 * Generated Prism Theme from VSCode .json Themes
 *
 * @see https://github.com/FormidableLabs/prism-react-renderer/blob/95025358684be04865669bb912e8cf6203c1a391/tools/themeFromVsCode/README.md
 */
declare const vscLightTheme: IPrismTheme;

/**
 * Generated Prism Theme from VSCode .json Themes
 *
 * @see https://github.com/FormidableLabs/prism-react-renderer/blob/d307f34360ecc4f0b4aadde4f72d09fd6dbf0132/src/themes/github.js
 */
declare const githubTheme: IPrismTheme;

declare function calcHeight(lineHeight: React.CSSProperties['lineHeight'], countOfLines: number): React.CSSProperties['height'];

declare function areSameArray(A1: unknown[] | undefined, A2: unknown[] | undefined): boolean;

declare const themeToDict: (language: string, theme: IPrismTheme) => IThemeDict;

/**
 * See https://github.com/FormidableLabs/prism-react-renderer/blob/d307f34360ecc4f0b4aadde4f72d09fd6dbf0132/src/utils/normalizeTokens.js
 */

declare const normalizeTokens: (tokens: TokenStream) => IToken[][];

declare const classes: {
    container: string;
    line: string;
    highlightLine: string;
    lineno: string;
    codes: string;
    codeWrapper: string;
    codeLine: string;
};

export { CodeHighlighter, HighlightContent, HighlightLinenos, HighlightTokens, ICodeHighlighterProps, IHighlightContentProps, IHighlightLinenosProps, IHighlightTokens, IHighlightTokensProps, ILineInputProps, ILineOutputProps, IPrismTheme, IThemeDict, IToken, ITokenInputProps, ITokenOutputProps, areSameArray, calcHeight, classes, CodeHighlighter as default, githubTheme, normalizeTokens, themeToDict, useHighlightTokens, vscDarkTheme, vscLightTheme };
